@@ -11,7 +11,10 @@ import pytest
 from imageio.core.request import InitializationError, Request
 from imageio.core.v3_plugin_api import PluginV3
 from imageio.plugins.pillow import PillowPlugin
-from PIL import Image, ImageSequence  # type: ignore
+from PIL import Image, ImageSequence, ImageOps, __version__  # type: ignore
+
+
+PILLOW_VERSION = tuple(int(x) for x in __version__.split("."))
 
 
 @pytest.mark.parametrize(
@@ -97,9 +100,7 @@ def test_png_quantization(test_images, tmp_path):
 
 
 def test_png_16bit(test_images, tmp_path):
-    # 16b bit images
     im = np.load(test_images / "chelsea.npy")[..., 0]
-
     iio.imwrite(
         tmp_path / "1.png",
         2 * im.astype(np.uint16),
@@ -115,17 +116,15 @@ def test_png_16bit(test_images, tmp_path):
     im2 = iio.imread(tmp_path / "2.png", plugin="pillow")
     assert im2.dtype == np.uint8
 
-    with pytest.warns(UserWarning):
+    if PILLOW_VERSION >= (10, 0, 0):
         im3 = iio.imread(tmp_path / "1.png", plugin="pillow")
+        assert im3.dtype == np.uint16
+    else:
+        # legacy behavior for pillow < v10.0.0
+        with pytest.warns(UserWarning):
+            im3 = iio.imread(tmp_path / "1.png", plugin="pillow")
 
-    assert im3.dtype == np.int32
-
-
-# Note: There was a test here referring to issue #352 and a `prefer_uint8`
-# argument that was introduced as a consequence This argument was default=true
-# (for backwards compatibility) in the legacy plugin with the recommendation to
-# set it to False. In the new API, we literally just wrap Pillow, so we match
-# their behavior. Consequentially this test was removed.
+        assert im3.dtype == np.int32
 
 
 @pytest.mark.needs_internet
@@ -191,7 +190,7 @@ def test_exif_orientation(test_images, tmp_path):
     # original image is has landscape format
     assert im.shape[0] < im.shape[1]
 
-    im_flipped = np.rot90(im, -1)
+    im_flipped = np.rot90(im, 1)
     exif_tag = Exif()
     exif_tag[274] = 6  # Set Orientation to 6
 
@@ -221,6 +220,12 @@ def test_exif_orientation(test_images, tmp_path):
 
     assert np.array_equal(im, im_reloaded)
 
+    # apply the transform with Pillow's exif_transpose
+    with Image.open(tmp_path / "chelsea_tagged.png") as f:
+        im_pillow = np.asarray(ImageOps.exif_transpose(f))
+
+    assert np.array_equal(im, im_pillow)
+
 
 def test_gif_rgb_vs_rgba(test_images):
     # Note: I don't understand the point of this test
@@ -255,14 +260,14 @@ def test_gif_gray(test_images, tmp_path):
     )
 
 
-def test_gif_fps_error(test_images, tmp_path):
+def test_gif_fps_warning(test_images, tmp_path):
     im = iio.imread(
         test_images / "newtonscradle.gif",
         plugin="pillow",
         mode="L",
     )
 
-    with pytest.raises(TypeError):
+    with pytest.warns(DeprecationWarning):
         iio.imwrite(
             tmp_path / "test.gif",
             im[..., 0],
@@ -355,6 +360,17 @@ def test_gif_list_write(test_images, tmp_path):
     assert im2.shape == (24, 30, 3)
 
 
+@pytest.mark.needs_internet
+def test_gif_first_p_frame():
+    # Bugfix: https://github.com/imageio/imageio/issues/1030
+    im = iio.imread(
+        "https://upload.wikimedia.org/wikipedia/commons/d/d3/Newtons_cradle_animation_book_2.gif",
+        plugin="pillow",
+        index=None,
+    )
+    assert im.shape == (36, 360, 480, 3)
+
+
 def test_legacy_exif_orientation(test_images, tmp_path):
     from PIL.Image import Exif
 
@@ -363,7 +379,7 @@ def test_legacy_exif_orientation(test_images, tmp_path):
     # original image is has landscape format
     assert im.shape[0] < im.shape[1]
 
-    im_flipped = np.rot90(im, -1)
+    im_flipped = np.rot90(im, 1)
     exif_tag = Exif()
     exif_tag[274] = 6  # Set Orientation to 6
 
@@ -394,6 +410,12 @@ def test_legacy_exif_orientation(test_images, tmp_path):
 
     assert np.array_equal(im, im_reloaded)
 
+    # apply the transform with Pillow's exif_transpose
+    with Image.open(tmp_path / "chelsea_tagged.png") as f:
+        im_pillow = np.asarray(ImageOps.exif_transpose(f))
+
+    assert np.array_equal(im, im_pillow)
+
 
 def test_incomatible_write_format(tmp_path):
     with pytest.raises(IOError):
@@ -683,3 +705,17 @@ def test_writable_output():
 
     frame = iio.imread(buffer, writeable_output=False, plugin="pillow")
     assert not frame.flags["WRITEABLE"]
+
+
+@pytest.mark.needs_internet
+def test_heif_remote():
+    url = "http://github.com/tigranbs/test-heic-images/raw/master/image4.heic"
+    im = iio.imread(url, plugin="pillow")
+    assert im.shape == (476, 700, 4)
+
+
+@pytest.mark.needs_internet
+def test_avif_remote():
+    url = "https://github.com/link-u/avif-sample-images/raw/master/fox.profile0.10bpc.yuv420.avif"
+    im = iio.imread(url, plugin="pillow")
+    assert im.shape == (800, 1204, 3)
